var twdExcludedClassName = "twd-excluded";
//
// used for checking if a URL is external or not
//
// eslint-disable-next-line no-unused-vars
function isURLExternal(url) {
    var host = window.location.hostname;
    var linkHost = function (url) {
        if (/^https?:\/\//.test(url)) { // Absolute URL.
            // The easy way to parse an URL, is to create <a> element. @see: https://gist.github.com/jlong/2428561
            var parser = document.createElement("a");
            parser.href = url;

            return parser.hostname;
        } else { // Relative URL.
            return window.location.hostname;
        }
    }(url);
    return host !== linkHost;
}

//
// adds CSS link to all iframe elements
//
// eslint-disable-next-line no-unused-vars
function addCSSLinkToAllIframeIfNotPresent(cssFilePath, linkElemID) {
    var iframeElemList = document.querySelectorAll("iframe");
    for (var index = 0, len = iframeElemList.length; index < len; index++) {
        // skip frames that are excluded
        if (iframeElemList[index]) {
            if (iframeElemList[index].classList.contains(twdExcludedClassName)) {
                // this element is excluded
                continue;
            }
        }
        if (iframeElemList[index] && iframeElemList[index].contentDocument) {
            // check if present, if present continue
            if (iframeElemList[index].contentDocument.querySelector("#" + linkElemID)) {
                continue;
            }
        }
        // create a new element every time
        var cssLink = document.createElement("link");
        cssLink.href = cssFilePath;
        cssLink.rel = "stylesheet";
        cssLink.type = "text/css";
        cssLink.id = linkElemID;
        try {
            console.log("iframe element is");
            console.log(iframeElemList[index]);
            if (iframeElemList[index] && iframeElemList[index].contentDocument && iframeElemList[index].contentDocument.head) {
                iframeElemList[index].contentDocument.head.appendChild(cssLink);
            }
        } catch (e) {
            // ignore the issue
            console.error(e);
        }
    }
}

//
// adds an internal CSS element to all iframe elements if not present
//
// eslint-disable-next-line no-unused-vars
function addInternalCSSToAllIframeIfNotPresent(cssText, styleElemID) {
    var iframeElemList = document.querySelectorAll("iframe");
    for (var index = 0, len = iframeElemList.length; index < len; index++) {
        // skip frames that are excluded
        if (iframeElemList[index]) {
            if (iframeElemList[index].classList.contains(twdExcludedClassName)) {
                // this element is excluded
                continue;
            }
        }
        // check if element is  present, if present continue
        if (iframeElemList[index] && iframeElemList[index].contentDocument) {
            if (iframeElemList[index].contentDocument.querySelector("#" + styleElemID)) {
                continue;
            }
        }
        // create a new element every time
        var styleElem = document.createElement("style");
        styleElem.innerText = cssText;
        styleElem.id = styleElemID;
        try {
            console.log("iframe element is");
            console.log(iframeElemList[index]);
            if (iframeElemList[index] && iframeElemList[index].contentDocument && iframeElemList[index].contentDocument.head) {
                iframeElemList[index].contentDocument.head.appendChild(styleElem);
            }
        } catch (e) {
            // ignore the issue
            console.error(e);
        }
    }
}

//
// removes an element with specific ID from iframe elements if present
//
// eslint-disable-next-line no-unused-vars
function removeElemIDFromAllIframeIfPresent(linkElemID) {
    var iframeElemList = document.querySelectorAll("iframe");
    for (var index = 0, len = iframeElemList.length; index < len; index++) {
        // skip frames that are excluded
        if (iframeElemList[index].classList.contains(twdExcludedClassName)) {
            // this element is excluded
            continue;
        }
        if (iframeElemList[index] && iframeElemList[index].contentDocument) {
            var elem = iframeElemList[index].contentDocument.querySelector("#" + linkElemID);
            if (elem) {
                elem.remove();
            }
        }
    }
}

//
// generates a list of various document types currently available on page
//
// eslint-disable-next-line no-unused-vars
function generateDocumentList() {
    // add normal documents
    var documentList = [];
    documentList.push(window.document);
    // add all iframe documents
    var iframeList = document.querySelectorAll("iframe");
    for (var index = 0, len = iframeList.length; index < len; index++) {
        try {
            if (iframeList[index].contentWindow.document) {
                // check if set then add to the list
                documentList.push(iframeList[index].contentWindow.document);
            }
        } catch (e) {
            // for cross origin error
        }
    }
    // add shadow roots to the document list
    var allNodes = document.querySelectorAll("*");
    for (var index1 = 0, len1 = allNodes.length; index1 < len1; index1++) {
        if (allNodes[index1].shadowRoot) {
            documentList.push(allNodes[index1].shadowRoot);
        }
    }
    return documentList;
}

//
// removes a specific class name from all doc type elements
//
// eslint-disable-next-line no-unused-vars
function removeClassNameFromAllDocTypeElem(className) {
    // eslint-disable-next-line no-undef
    var docList = generateDocumentList();
    for (var index = 0, len = docList.length; index < len; index++) {
        var withBackgroundImg = docList[index].querySelectorAll("." + className);
        // remove from bottom
        for (var index2 = withBackgroundImg.length; index2 >= 0; index2--) {
            if (withBackgroundImg[index2]) {
                withBackgroundImg[index2].classList.remove(className);
            }
        }
    }
}

//
// checks if an element has class
//
// eslint-disable-next-line no-unused-vars
function hasClass(element, className) {
    if (!element || !className) {
        return false;
    }
    return element.classList.contains(className);
}

//
// adds a new class to an element
//
// eslint-disable-next-line no-unused-vars
function addClass(element, className) {
    if (!element || !className) {
        return;
    }
    return element.classList.add(className);
}

//
// gets CSS property value
//
// eslint-disable-next-line no-unused-vars
function getCSSPropertyVal(computedStyle, property) {
    var cssProperty = null;
    // check if the computed style is set
    if (computedStyle) {
        // get the computed style
        try {
            cssProperty = computedStyle.getPropertyCSSValue(property);
        } catch (exception) {
            // (fails in newer versions of Chrome)
            cssProperty = computedStyle.getPropertyValue(property);
        }
    }
    return cssProperty;
}

//
// gets CSS text
//
// eslint-disable-next-line no-unused-vars
function getCSSText(property) {
    if (property) {
        if (property.cssText) {
            return property.cssText;
        }
        return property;
    }
    return null;
}
